/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen.plugins.entitybean;

import java.util.Observer;
import java.util.Observable;
import middlegen.Column;
import middlegen.javax.JavaColumn;
import middlegen.Table;
import middlegen.Util;
import middlegen.javax.Sql2Java;
import middlegen.util.BooleanNode;

/**
 * Represents a column in EJB 1.1 Entity Beans
 *
 * @author <a href="mailto:aslak.hellesoy@bekk.no">Aslak Hellesy</a>
 * @created 3. oktober 2001
 * @version $Id: Entity11Column.java,v 1.1 2004/08/20 06:53:42 manu Exp $
 */
public class Entity11Column extends JavaColumn implements Observer {
   /**
    * @todo-javadoc Describe the column
    */
   private BooleanNode _isRemoteSet;
   /**
    * @todo-javadoc Describe the column
    */
   private BooleanNode _isRemoteGet;

   /**
    * Get static reference to Log4J Logger
    */
   private static org.apache.log4j.Category _log = org.apache.log4j.Category.getInstance(Entity11Column.class.getName());


   /**
    * Describe what the Entity11Column constructor does
    *
    * @param subject Describe what the parameter does
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   public Entity11Column(Column subject) {
      super(subject);
   }


   /**
    * Describe what the setRemoteSet constructor does
    *
    * @param flag Describe what the parameter does
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   public void setRemoteSet(boolean flag) {
      setPrefsValue("remote-set", Util.string(flag));
      if (flag != _isRemoteSet.isCompletelyTrue()) {
         _isRemoteSet.setValue(flag);
      }
   }


   /**
    * Describe what the setRemoteGet constructor does
    *
    * @param flag Describe what the parameter does
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   public void setRemoteGet(boolean flag) {
      setPrefsValue("remote-get", Util.string(flag));
      if (flag != _isRemoteGet.isCompletelyTrue()) {
         _isRemoteGet.setValue(flag);
      }
   }


   /**
    * Gets the RemoteSet attribute of the Entity11Column object
    *
    * @return The RemoteSet value
    */
   public boolean isRemoteSet() {
      return _isRemoteSet.isCompletelyTrue();
   }


   /**
    * Gets the RemoteGet attribute of the Entity11Column object
    *
    * @return The RemoteGet value
    */
   public boolean isRemoteGet() {
      return _isRemoteGet.isCompletelyTrue();
   }


   /**
    * Gets the RemoteGet attribute of the Entity11Column object
    *
    * @return The RemoteGet value
    */
   public BooleanNode getRemoteGet() {
      return _isRemoteGet;
   }


   /**
    * Gets the RemoteSet attribute of the Entity11Column object
    *
    * @return The RemoteSet value
    */
   public BooleanNode getRemoteSet() {
      return _isRemoteSet;
   }


   /**
    * Describe what the method does
    *
    * @param o Describe what the parameter does
    * @param arg Describe what the parameter does
    * @todo-javadoc Write javadocs for method
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for method parameter
    */
   public void update(Observable o, Object arg) {
      if (o == _isRemoteSet) {
         setRemoteSet(_isRemoteSet.isCompletelyTrue());
      }
      else if (o == _isRemoteGet) {
         setRemoteGet(_isRemoteGet.isCompletelyTrue());
      }
   }


   /**
    * Overrides method in superclass. Called from super.init()
    */
   protected void setJavaType() {

      //System.out.println("bc // setJavaType"  + getSqlName()) ;

      if (getPrefsValue("java-type") != null) {
         setJavaType(getPrefsValue("java-type"));
      }
      else {
         if (isPk() || isFk() || (isNullable() && getSqlType() != 4)) {
            // can't use primitive type when we're a simple pk. avoid primitives for fks and nullable too
            setJavaType(Sql2Java.getPreferredJavaTypeNoPrimitives(getSqlType(), getSize(), getDecimalDigits()));
         }
         else {

            setJavaType(Sql2Java.getPreferredJavaType(getSqlType(), getSize(), getDecimalDigits()));
         }
      }
   }


   /**
    * Gets the RemoteEnabled attribute of the Entity11Column object
    *
    * @return The RemoteEnabled value
    */
   protected boolean isRemoteEnabled() {
      return true;
   }


   /**
    * Returns true if the plugin has the readonly flag set to true.
    *
    * @return The ReadonlyDefault value
    */
   protected boolean isReadonlyDefault() {
      return ((Entity11Plugin)getPlugin()).isReadonly();
   }


   /**
    * Describe what the method does
    *
    * @todo-javadoc Write javadocs for method
    */
   protected void init() {
      super.init();

      // we initialise them to false if we're pk or fk unless there is a value in prefs
      Entity11Table table = (Entity11Table)getTable();
      //	System.out.println("Entity11Column.init() TABLE:" + table + " I am " + getSqlName());
      // these will be overriden
      _isRemoteSet = table.getRemote().createChild(true);
      _isRemoteGet = table.getRemote().createChild(true);
      // We want to know when the view-type changes
      _isRemoteSet.addObserver(this);
      _isRemoteGet.addObserver(this);

      // Don't enable setters for pk fields or fk fields. They are read-only.
      boolean computedRemoteSet = !(isPk() || isFk()) && isRemoteEnabled() && !isReadonlyDefault();
      if (getPrefsValue("remote-set") != null) {
         boolean prefsRemoteSet = Util.bool(getPrefsValue("remote-set"));
         if (computedRemoteSet == false && prefsRemoteSet == true) {
            _log.warn("WARNING: The " + getPlugin().getName() + " is configured with readonly=\"true\", " +
                  "but the prefs settings for the column " + table.getSqlName() + "." + getSqlName() +
                  " indicates that a remote setter for this column should be generated. " +
                  "Middlegen will use the prefs setting, so the generated " +
                  table.getRemoteClassName() + "'s set" + getCapitalisedVariableName() + " method " +
                  "will exist, causing the bean NOT to be read only. " +
                  "If the bean should really be read-only, change the settings in the gui or in the prefs file directly."
                  );
         }
         setRemoteSet(prefsRemoteSet);
      }
      else {
         setRemoteSet(computedRemoteSet);
      }
      if (getPrefsValue("remote-get") != null) {
         setRemoteGet(Util.bool(getPrefsValue("remote-get")));
      }
      else {
         // Enable getters for all remote fields (since we don't have CMR on remote)
         setRemoteGet(isRemoteEnabled());
      }

      //	System.out.println("isPk():" + isPk() + "isFk():" + isFk() + ":" + getSqlName());
   }
}

