/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen.plugins.entitybean;

import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;
import middlegen.javax.JavaPlugin;
import middlegen.Table;
import middlegen.Util;
import middlegen.MiddlegenException;
import middlegen.FileProducer;
import middlegen.util.BooleanNode;
import middlegen.predicate.ant.DynamicPredicate;
import middlegen.predicates.column.NotKey;
import middlegen.predicates.column.ColumnPredicateFactory;

import org.apache.commons.collections.Predicate;

/**
 * This plugin generates Enterprise Java Beans of type EntityBean conforming to
 * the EJB 1.1 specification.
 *
 * @author <a href="mailto:aslak.hellesoy@bekk.no">Aslak Hellesy</a>
 * @created 22. mars 2002
 * @todo add support for customisation of the names, while still providing
 *      defaults
 * @todo maybe this class should be a subtask?
 */
public abstract class Entity11Plugin extends JavaPlugin {

   /**
    * @todo-javadoc Describe the field
    */
   private final ArrayList _servers = new ArrayList();
   /**
    * Weblogic bean containing server specific settings
    */
   private Weblogic _weblogic;
   /**
    * JBoss bean containing server specific settings
    */
   private JBoss _jboss;
   /**
    * JOnAS bean containing server specific settings
    */
   private JOnAS _jonas;
   /**
    * MVCSoft bean containing server specific settings
    */
   private MVCSoft _mvcsoft;

   /**
    * @todo-javadoc Describe the field
    */
   private SequenceBlock _sequenceBlock;

   /**
    * @todo-javadoc Describe the column
    */
   private String _beansuffix = "Bean";
   /**
    * @todo-javadoc Describe the column
    */
   private String _remotesuffix = "Remote";
   /**
    * @todo-javadoc Describe the column
    */
   private String _remotehomesuffix = "Home";
   /**
    * @todo-javadoc Describe the column
    */
   private String _datasuffix = "Data";
   /**
    * @todo-javadoc Describe the column
    */
   private String _pksuffix = "PK";
   /**
    * @todo-javadoc Describe the column
    */
   private String _utilsuffix = "Util";
   /**
    * @todo-javadoc Describe the column
    */
   private String _interfacePackage;
   /**
    * @todo-javadoc Describe the column
    */
   private final BooleanNode _isRemote = BooleanNode.createRoot(false);

   /**
    * @todo-javadoc Describe the column
    */
   private String _jndiPrefix;

   /**
    * @todo-javadoc Describe the column
    */
   private boolean _pkClass = false;

   /**
    * @todo-javadoc Describe the field
    */
   private boolean _dataobject = false;

   /**
    * @todo-javadoc Describe the field
    */
   private boolean _valueobject = false;

   /**
    * @todo-javadoc Describe the field
    */
   private boolean _sessionFacade = false;

   /**
    * @todo-javadoc Describe the field
    */
   private boolean _isReadonly = false;

   /**
    * @todo-javadoc Describe the field
    */
   private DynamicPredicate _finders = null;

   /**
    * @todo-javadoc Describe the field
    */
   private boolean _guid = false;

   /**
    * OracleSequence bean containing settings for using Oracle sequences
    */
   private OracleSequence _oracleSequence;

   /**
    * Get static reference to Log4J Logger
    */
   private static org.apache.log4j.Category _log = org.apache.log4j.Category.getInstance(Entity11Plugin.class.getName());


   /**
    * Describe what the Entity11Plugin constructor does
    *
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   protected Entity11Plugin() {
      super();

      // warn if datasourceJNDIName was not specified
      //if (getMiddlegen().getMiddlegenTask().getDataSourceJNDIName() == null) {
      //	System.err.println("Warning: You should define datasourceJNDIName in the Ant task");
      //}
   }


   /**
    * Set it to true to get automatic guid generation
    *
    * @param guid The new Guid value
    * @todo-javadoc Write javadocs for exception
    * @throws MiddlegenExcdption if not supported
    */
   public void setGuid(boolean guid) {
      _guid = guid;
   }


   /**
    * Sets the Dataobject attribute of the Entity11Plugin object
    *
    * @param dataobject The new Dataobject value
    */
   public void setDataobject(boolean dataobject) {
      _dataobject = dataobject;
   }


   /**
    * Sets the Valueobject attribute of the Entity11Plugin object
    *
    * @param valueobject The new Valueobject value
    */
   public void setValueobject(boolean valueobject) {
      _valueobject = valueobject;
   }


   /**
    * Sets the Sessionfacade attribute of the Entity11Plugin object
    *
    * @param sessionFacade The new Sessionfacade value
    */
   public void setSessionfacade(boolean sessionFacade) {
      _sessionFacade = sessionFacade;
   }


   /**
    * If this is set to false, a primary key class will never be generated
    * unless a table's primary key consists of more than one column. If a
    * table's primary key consists of several columns, a compound primary key
    * class will be generated regardless of this setting.
    *
    * @param flag The new Generatepk value
    */
   public void setPkclass(boolean flag) {
      _pkClass = flag;
   }


   /**
    * @param flag The new Generatepk value
    * @deprecated because the name is confusing with respect to database pk
    *      generation. Use setGeneratepkclass instead.
    */
   public void setGeneratepk(boolean flag) {
      _log.warn("The generatepk attribute is deprecated and will be removed in a future version " +
            "of Middlegen. Please use the pkclass attribute instead."
            );
      setPkclass(flag);
   }


   /**
    * Sets the Readonly attribute of the Entity11Plugin object
    *
    * @param flag The new Readonly value
    */
   public void setReadonly(boolean flag) {
      _isReadonly = flag;
   }


   /**
    * Sets the Jndiprefix attribute of the Entity11Plugin object
    *
    * @param jndiPrefix The new Jndiprefix value
    */
   public void setJndiprefix(String jndiPrefix) {
      _jndiPrefix = jndiPrefix;
   }


   /**
    * Sets the PackageName attribute of the EntityCMPClassType object
    *
    * @param interfacePackage The new Interfacepackage value
    */
   public void setInterfacepackage(String interfacePackage) {
      if (interfacePackage.trim().equals("")) {
         throw new IllegalArgumentException("Can't use empty String for interfacepackage!");
      }
      _interfacePackage = interfacePackage;
   }


   /**
    * Sets the Beansuffix attribute of the EntityClassType object
    *
    * @param s The new Beansuffix value
    */
   public void setBeansuffix(String s) {
      _beansuffix = s;
   }


   /**
    * Sets the Remotesuffix attribute of the EntityClassType object
    *
    * @param s The new Remotesuffix value
    */
   public void setRemotesuffix(String s) {
      _remotesuffix = s;
   }


   /**
    * Sets the Remotehomesuffix attribute of the EntityClassType object
    *
    * @param s The new Remotehomesuffix value
    */
   public void setRemotehomesuffix(String s) {
      _remotehomesuffix = s;
   }


   /**
    * Sets the Datasuffix attribute of the EntityClassType object
    *
    * @param s The new Datasuffix value
    */
   public void setDatasuffix(String s) {
      _datasuffix = s;
   }


   /**
    * Sets the Pksuffix attribute of the EntityClassType object
    *
    * @param s The new Pksuffix value
    */
   public void setPksuffix(String s) {
      _pksuffix = s;
   }


   /**
    * Gets the Guid attribute of the Entity11Plugin object
    *
    * @return The Guid value
    */
   public boolean isGuid() {
      return _guid;
   }


   /**
    * Gets the Pkgen attribute of the Weblogic object
    *
    * @return The Pkgen value
    */
   public boolean isPkgen() {
      // Standard sequence block pattern. Works for all containers.
      boolean sequenceBlock = getSequenceblock() != null;

      // Oracle sequence block pattern. Works for example with
      // JBoss 3.0+ and Oracle
      boolean oraclesequence = getOracleSequence() != null;

      // WebLogic proprietary pk gen
      boolean weblogic = false;
      if (getWeblogic() != null) {
         weblogic = getWeblogic().getGeneratorType() != null;
      }

      return sequenceBlock || weblogic || oraclesequence;
   }


   /**
    * Gets the Weblogic attribute of the Entity11Plugin object
    *
    * @return The Weblogic value
    */
   public Weblogic getWeblogic() {
      return _weblogic;
   }


   /**
    * Gets the JBoss attribute of the Entity11Plugin object
    *
    * @return The JBoss value
    */
   public JBoss getJBoss() {
      return _jboss;
   }


   /**
    * Gets the JOnAS attribute of the Entity11Plugin object
    *
    * @return The JOnAS value
    */
   public JOnAS getJOnAS() {
      return _jonas;
   }


   /**
    * Gets the MVCSoft attribute of the Entity11Plugin object
    *
    * @return The MVCSoft plugin object
    */
   public MVCSoft getMvcsoft() {
      return _mvcsoft;
   }


   /**
    * Gets the Readonly attribute of the Entity11Plugin object
    *
    * @return The Readonly value
    */
   public boolean isReadonly() {
      return _isReadonly;
   }


   /**
    * Gets the Dataobject attribute of the Entity11Plugin object
    *
    * @return The Dataobject value
    */
   public boolean getDataobject() {
      return _dataobject;
   }


   /**
    * Gets the Valueobject attribute of the Entity11Plugin object
    *
    * @return The Valueobject value
    */
   public boolean getValueobject() {
      return _valueobject;
   }


   /**
    * Gets the Sessionfacade attribute of the Entity11Plugin object
    *
    * @return The Sessionfacade value
    */
   public boolean getSessionfacade() {
      return _sessionFacade;
   }


   /**
    * Returns true if a pk class should be generated
    *
    * @return The GeneratePk value
    */
   public boolean isPkClass() {
      return _pkClass;
   }


   /**
    * Gets the JNDIPrefix attribute of the Entity11Plugin object
    *
    * @return The JNDIPrefix value
    */
   public String getJNDIPrefix() {
      return _jndiPrefix;
   }


   /**
    * Gets the Beansuffix attribute of the Entity11Plugin object
    *
    * @return The Beansuffix value
    */
   public String getBeansuffix() {
      return _beansuffix;
   }


   /**
    * Gets the Utilsuffix attribute of the Entity11Plugin object
    *
    * @return The Utilsuffix value
    */
   public String getUtilsuffix() {
      return _utilsuffix;
   }


   /**
    * Gets the Remotesuffix attribute of the Entity11Plugin object
    *
    * @return The Remotesuffix value
    */
   public String getRemotesuffix() {
      return _remotesuffix;
   }


   /**
    * Gets the Remotehomesuffix attribute of the Entity11Plugin object
    *
    * @return The Remotehomesuffix value
    */
   public String getRemotehomesuffix() {
      return _remotehomesuffix;
   }


   /**
    * Gets the Datasuffix attribute of the Entity11Plugin object
    *
    * @return The Datasuffix value
    */
   public String getDatasuffix() {
      return _datasuffix;
   }


   /**
    * Gets the Pksuffix attribute of the Entity11Plugin object
    *
    * @return The Pksuffix value
    */
   public String getPksuffix() {
      return _pksuffix;
   }


   /**
    * Gets the ColumnDecoratorClass attribute of the Entity11Plugin object
    *
    * @return The ColumnDecoratorClass value
    */
   public Class getColumnDecoratorClass() {
      return Entity11Column.class;
   }


   /**
    * Gets the TableDecoratorClass attribute of the Entity11Plugin object
    *
    * @return The TableDecoratorClass value
    */
   public Class getTableDecoratorClass() {
      return Entity11Table.class;
   }


   /**
    * Gets the InterfacePackageName attribute of the EntityCMPClassType object
    *
    * @return The InterfacePackageName value
    */
   public String getInterfacePackage() {
      // this won't work if the user sets package to "", but that's not very likely for EJBs...
      return _interfacePackage != null ? _interfacePackage : getPackage() + ".interfaces";
   }


   /**
    * Gets the Remote attribute of the Entity11Plugin object
    *
    * @return The Remote value
    */
   public BooleanNode getRemote() {
      return _isRemote;
   }


   /**
    * Gets the FindersPredicate attribute of the Entity11Plugin object
    *
    * @return The FindersPredicate value
    */
   public Predicate getFindersPredicate() {
      if (_finders != null) {
         return _finders.getPredicate();
      }
      else {
         // Default behaviour (if there is no <finders/> element)
         // Don't generate finders for pk fields (They are covered by the standard findByPrimaryKey)
         // Don't generate finders for fk fields (They are covered by the CMR methods)
         return NotKey.getInstance();
      }
   }


   /**
    * Gets the Sequenceblock attribute of the Entity11Plugin object
    *
    * @return The Sequenceblock value
    */
   public SequenceBlock getSequenceblock() {
      return _sequenceBlock;
   }


   /**
    * Gets the OracleSequence attribute of the Entity11Plugin object
    *
    * @return The OracleSequence value
    */
   public OracleSequence getOracleSequence() {
      return _oracleSequence;
   }


   /**
    * Describe what the method does
    *
    * @return Describe the return value
    * @todo-javadoc Write javadocs for method
    * @todo-javadoc Write javadocs for return value
    */
   public DynamicPredicate createFinders() {
      ColumnPredicateFactory predicateFactory = new ColumnPredicateFactory();
      _finders = new DynamicPredicate(predicateFactory);
      return _finders;
   }


   /**
    * Describe the method
    *
    * @param weblogic Describe the method parameter
    * @todo-javadoc Describe the method
    * @todo-javadoc Describe the method parameter
    */
   public void addWeblogic(Weblogic weblogic) {
      _weblogic = weblogic;
      _weblogic.setPlugin(this);
      _servers.add(_weblogic);
   }


   /**
    * Describe the method
    *
    * @param jboss Describe the method parameter
    * @todo-javadoc Describe the method
    * @todo-javadoc Describe the method parameter
    */
   public void addJBoss(JBoss jboss) {
      _jboss = jboss;
      _jboss.setPlugin(this);
      _servers.add(_jboss);
   }


   /**
    * Describe the method
    *
    * @param jonas Describe the method parameter
    * @todo-javadoc Describe the method
    * @todo-javadoc Describe the method parameter
    */
   public void addJOnAS(JOnAS jonas) {
      _jonas = jonas;
      _jonas.setPlugin(this);
      _servers.add(_jonas);
   }


   /**
    * This method is used by Ant to associate the MVCSoft server support bean
    * with the plug-in.
    *
    * @param mvcsoft MVCSoft server support bean
    */
   public void addMvcsoft(MVCSoft mvcsoft) {
      _mvcsoft = mvcsoft;
      _mvcsoft.setPlugin(this);
      _servers.add(_mvcsoft);
   }



   /**
    * Describe the method
    *
    * @param sequenceBlock Describe the method parameter
    * @todo-javadoc Describe the method
    * @todo-javadoc Describe the method parameter
    */
   public void addSequenceblock(SequenceBlock sequenceBlock) {
      _sequenceBlock = sequenceBlock;

      // the sequenceBlock needs us to access default values for package
      _sequenceBlock.setPlugin(this);
   }


   /**
    * Adds the &lt;oraclesequence/&gt; subelement.
    *
    * @param oracleSequence the OracleSequence subelement
    */
   public void addOracleSequence(OracleSequence oracleSequence) {
      _oracleSequence = oracleSequence;
      // the oracleSequence needs us to access default values for package
      _oracleSequence.setPlugin(this);
   }


   /**
    * Describe what the method does
    *
    * @exception MiddlegenException Describe the exception
    * @todo-javadoc Write javadocs for method
    * @todo-javadoc Write javadocs for exception
    */
   public void validate() throws MiddlegenException {
      super.validate();
      if (_finders != null && _finders.getPredicate() == null) {
         throw new MiddlegenException("The <finders/> element can't be empty!");
      }
      if ((getSequenceblock() != null) && (getOracleSequence() != null)) {
         throw new MiddlegenException("Elements <sequenceblock/> and <oraclesequence can not be present in one <cmp20>-Element!");
      }
      for (Iterator servers = _servers.iterator(); servers.hasNext(); ) {
         Server server = (Server)servers.next();
         server.validate();
      }
   }


   /**
    * Describe what the method does
    *
    * @todo-javadoc Write javadocs for method
    */
   protected void registerFileProducers() {
      super.registerFileProducers();
      if (getSequenceblock() != null) {
         addConfiguredFileproducer(new FileProducer(getDestinationDir(), getSequenceblock().getPackage().replace('.', '/') + "/SequenceSessionBean.java", getClass().getResource("sequence-session-bean.vm")));
         addConfiguredFileproducer(new FileProducer(getDestinationDir(), getSequenceblock().getPackage().replace('.', '/') + "/SequenceBean.java", getClass().getResource("sequence-bean.vm")));
      }
      if (getOracleSequence() != null) {
         addConfiguredFileproducer(new FileProducer(getDestinationDir(), getOracleSequence().getPackage().replace('.', '/') + "/OracleSequenceSessionBean.java", getClass().getResource("oracle-sequence-session-bean.vm")));
      }
   }
}
