/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen.plugins.entitybean.swing;

import middlegen.*;
import middlegen.javax.JavaTable;
import middlegen.plugins.entitybean.Entity20Column;
import middlegen.plugins.entitybean.Entity20Table;
import middlegen.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.*;
import javax.swing.event.*;

/**
 * This panel renders and modifies columns
 *
 * @author Aslak Hellesy
 * @created 3. oktober 2001
 * @version
 */
public class JEntityTableSettingsPanel extends JTableSettingsPanel {

   /**
    * @todo-javadoc Describe the column
    */
   ActionListener _pkListener =
      new ActionListener() {
         public void actionPerformed(ActionEvent evt) {
            // if 2nd element is selected, it's the Generated pk class
            _currentTable.setGeneratePkClass(_primaryKeyTypeComboBox.getSelectedIndex() == 0);
         }
      };

   /**
    * @todo-javadoc Describe the field
    */
   ActionListener _extListener = null;

   /**
    * @todo-javadoc Describe the field
    */
   ActionListener _integListener = null;

   /**
    * @todo-javadoc Describe the column
    */
   private final JLabel _ejbNameLabel = new JLabel("Logical EJB name");

   /**
    * @todo-javadoc Describe the column
    */
   private final JTextField _ejbNameField = new JTextField();

   /**
    * @todo-javadoc Describe the column
    */
   private final JLabel _viewTypeLabel = new JLabel("View type");

   /**
    * @todo-javadoc Describe the column
    */
   private final BooleanNodeCheckBox _localCheckBox = new BooleanNodeCheckBox(
         "Local", null);

   /**
    * @todo-javadoc Describe the column
    */
   private final BooleanNodeCheckBox _remoteCheckBox = new BooleanNodeCheckBox(
         "Remote", null);

   /**
    * @todo-javadoc Describe the column
    */
   private final JLabel _primaryKeyTypeLabel = new JLabel("Primary key class");

   /**
    * @todo-javadoc Describe the field
    */
   private final JLabel _extendsTableLabel = new JLabel("Extends");

   /**
    * @todo-javadoc Describe the field
    */
   private final DefaultComboBoxModel _extendsTableComboModel = new DefaultComboBoxModel();

   /**
    * @todo-javadoc Describe the field
    */
   private final JComboBox _extendsTableComboBox = new JComboBox(
         _extendsTableComboModel);

   /**
    * @todo-javadoc Describe the field
    */
   private final JLabel _integriteTableLabel = new JLabel("Intgrit");

   /**
    * @todo-javadoc Describe the field
    */
   private final DefaultComboBoxModel _integriteTableComboModel = new DefaultComboBoxModel();

   /**
    * @todo-javadoc Describe the field
    */
   private final JComboBox _integriteTableComboBox = new JComboBox(
         _integriteTableComboModel);

   /**
    * @todo-javadoc Describe the column
    */
   private final DefaultComboBoxModel _primaryKeyComboModel = new DefaultComboBoxModel();

   /**
    * @todo-javadoc Describe the column
    */
   private final JComboBox _primaryKeyTypeComboBox = new JComboBox(
         _primaryKeyComboModel);

   /**
    * @todo-javadoc Describe the column
    */
   private final DocumentListener _documentListener =
      new DocumentListener() {
         public void changedUpdate(DocumentEvent event) {
            update();
         }


         public void insertUpdate(DocumentEvent event) {
            update();
         }


         public void removeUpdate(DocumentEvent event) {
            update();
         }


         private void update() {
            updateTable(_ejbNameField.getText());
         }
      };

   /**
    * @todo-javadoc Describe the column
    */
   private Entity20Table _currentTable;

   /**
    * @todo-javadoc Describe the field
    */
   private static org.apache.log4j.Category _log = org.apache.log4j.Category.getInstance(JEntityTableSettingsPanel.class.getName());


   /**
    * Describe what the JTableSettingsPanel constructor does
    *
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for constructor
    */
   public JEntityTableSettingsPanel() {

      super();

      GridLayout layout = new GridLayout(0, 2, 4, 8);
      setLayout(layout);

      add(_ejbNameLabel);
      add(_ejbNameField);
      add(_primaryKeyTypeLabel);
      add(_primaryKeyTypeComboBox);
      add(_viewTypeLabel);

      //
      JPanel getterPanel = new JPanel();
      getterPanel.add(_localCheckBox);
      getterPanel.add(_remoteCheckBox);
      add(getterPanel);

      // add extend table ...
      add(_extendsTableLabel);
      add(_extendsTableComboBox);

      // add intgrit champ.
      add(_integriteTableLabel);
      add(_integriteTableComboBox);

      // register listeners on text column and combo that will update borders
      _ejbNameField.getDocument().addDocumentListener(_documentListener);

   }


   /**
    * @param table The new Table value
    */
   public void setTable(Table table) {

      _log.debug("entre danns setTable de JEntityTableSettingsPanel ...");

      _currentTable = (Entity20Table)table;

      _ejbNameField.setText(_currentTable.getBeanName());
      _remoteCheckBox.setBooleanNode(_currentTable.getRemote());
      _localCheckBox.setBooleanNode(_currentTable.getLocal());

      // remove action listener to avoid invoking it while building the combo
      // model
      _primaryKeyTypeComboBox.removeActionListener(_pkListener);
      _primaryKeyComboModel.removeAllElements();

      _primaryKeyComboModel.addElement(_currentTable.getGeneratedPkClassName());
      String simplePkClassName = _currentTable.getSimplePkClassName();
      if (simplePkClassName != null) {
         _primaryKeyComboModel.addElement(simplePkClassName);
      }
      _primaryKeyTypeComboBox.setSelectedItem(_currentTable.getPkClassName());

      // add listener on pk combo
      _primaryKeyTypeComboBox.addActionListener(_pkListener);

      //  positionement table extends pour hritage

      if (_extListener != null) {
         _extendsTableComboBox.removeActionListener(_extListener);
      }

      _extendsTableComboModel.removeAllElements();
      Iterator it = _currentTable.getRelationshipRoles().iterator();
      while (it.hasNext()) {
         RelationshipRole role = (RelationshipRole)it.next();
         if (role.isTargetMany()) {
            continue;
         }
         if (_extendsTableComboModel.getSize() == 0) {
            _extendsTableComboModel.addElement("");
         }
         _log.debug("Table : " + role.getTarget().getName());
         _extendsTableComboModel.addElement(role.getTarget().getName());
      }
      _log.debug("extend table : " + _currentTable.getExtentTableName()
             + " for " + _currentTable.getName());

      if (_currentTable.getExtentTableName() != null) {
         _extendsTableComboBox.setSelectedItem(_currentTable.getExtentTableName());
      }

      _extListener = new JEntityTableSettingsPanel_extentTable_actionAdapter(
            this);
      _extendsTableComboBox.addActionListener(_extListener);

      // positionement champ INTEGRITE
      if (_integListener != null) {
         _integriteTableComboBox.removeActionListener(_integListener);
      }

      _integriteTableComboModel.removeAllElements();

      Collection obj = _currentTable.getMandatoryBasicColumns();
      it = obj.iterator();
      while (it.hasNext()) {
         Entity20Column en20c = (Entity20Column)it.next();
         _log.debug(" colums mandatory : " + en20c.getClass().getName());
         if (_integriteTableComboModel.getSize() == 0) {
            _integriteTableComboModel.addElement("");
         }
         _integriteTableComboModel.addElement(en20c.getVariableName());
      }
      if (_currentTable.getIntegriteTable() != null) {
         _integriteTableComboBox.setSelectedItem(_currentTable.getIntegriteTable());
      }

      _integListener = new JEntityTableSettingsPanel_integTable_actionAdapter(
            this);
      _integriteTableComboBox.addActionListener(_integListener);

   }


   /**
    * Gets the PreferredSize attribute of the JTableSettingsPanel object
    *
    * @return The PreferredSize value
    */
   public Dimension getPreferredSize() {
      return getMinimumSize();
   }


   /**
    * Describe what the method does
    *
    * @param e Describe what the parameter does
    * @todo-javadoc Write javadocs for method
    * @todo-javadoc Write javadocs for method parameter
    */
   void extentTable_actionPerformed(ActionEvent e) {

      _log.debug(e.getActionCommand() + " "
             + _extendsTableComboBox.getSelectedIndex() + " "
             + _extendsTableComboBox.getSelectedItem());
      if (e.getActionCommand().equalsIgnoreCase("comboBoxChanged") == true) {
         _currentTable.setExtentTableName((String)_extendsTableComboBox.getSelectedItem());
         //			posExtentTable();
      }

   }


   /**
    * Describe what the method does
    *
    * @param e Describe what the parameter does
    * @todo-javadoc Write javadocs for method
    * @todo-javadoc Write javadocs for method parameter
    */
   void integTable_actionPerformed(ActionEvent e) {

      _log.debug(e.getActionCommand() + " "
             + _integriteTableComboBox.getSelectedIndex() + " "
             + _integriteTableComboBox.getSelectedItem());
      if (e.getActionCommand().equalsIgnoreCase("comboBoxChanged") == true) {
         _currentTable.setIntegriteTable((String)_integriteTableComboBox.getSelectedItem());
         //			posExtentTable();
      }

   }


   /**
    * Describe what the method does
    *
    * @param beanName Describe what the parameter does
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for method
    */
   private void updateTable(String beanName) {
      _currentTable.setBeanName(beanName);
   }

   //	private void posExtentTable() {
   //
   //		// rechercher dans les realtions la table associe
   //		Iterator it = _currentTable.getRelationshipRoles().iterator();
   //		RelationshipRole role = null;
   //		while (it.hasNext()) {
   //			role = (RelationshipRole) it.next();
   //
   //			if (role.isTargetMany())
   //				continue;
   //
   //			if (!role.getTarget().getName().equalsIgnoreCase(
   //					_currentTable.getExtentTableName()))
   //				continue;
   //
   //			Table t = role.getTarget(_currentTable.getPlugin());
   //
   //			// Entity20Table en20 = (Entity20Table) t;
   //			// _log.debug(" package" + en20.getPackage());
   //
   //			_currentTable.setExtentTable(t);
   //
   //			break;
   //
   //		}
   //
   //	}
}

/**
 * Describe what this class does
 *
 * @author Cnedi
 * @created 19 aot 2004
 * @todo-javadoc Write javadocs
 */
class JEntityTableSettingsPanel_extentTable_actionAdapter implements
      java.awt.event.ActionListener {


   /**
    * @todo-javadoc Describe the field
    */
   JEntityTableSettingsPanel adaptee;


   /**
    * Describe what the JEntityTableSettingsPanel_extentTable_actionAdapter
    * constructor does
    *
    * @param adaptee Describe what the parameter does
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   JEntityTableSettingsPanel_extentTable_actionAdapter(
         JEntityTableSettingsPanel adaptee) {
      this.adaptee = adaptee;
   }


   /**
    * Describe what the method does
    *
    * @param e Describe what the parameter does
    * @todo-javadoc Write javadocs for method
    * @todo-javadoc Write javadocs for method parameter
    */
   public void actionPerformed(ActionEvent e) {
      adaptee.extentTable_actionPerformed(e);
   }
}

/**
 * Describe what this class does
 *
 * @author Cnedi
 * @created 19 aot 2004
 * @todo-javadoc Write javadocs
 */
class JEntityTableSettingsPanel_integTable_actionAdapter implements
      java.awt.event.ActionListener {


   /**
    * @todo-javadoc Describe the field
    */
   JEntityTableSettingsPanel adaptee;


   /**
    * Describe what the JEntityTableSettingsPanel_integTable_actionAdapter
    * constructor does
    *
    * @param adaptee Describe what the parameter does
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   JEntityTableSettingsPanel_integTable_actionAdapter(
         JEntityTableSettingsPanel adaptee) {
      this.adaptee = adaptee;
   }


   /**
    * Describe what the method does
    *
    * @param e Describe what the parameter does
    * @todo-javadoc Write javadocs for method
    * @todo-javadoc Write javadocs for method parameter
    */
   public void actionPerformed(ActionEvent e) {
      adaptee.integTable_actionPerformed(e);
   }
}
