package ejb.flights.entity;

/**
 * This class is part of Middlegen airlines, and it is a CMP EJB accessing the
 * flights table.
 * 
 * @author <a href="http://boss.bekk.no/boss/middlegen/">Middlegen </a>
 * 
 * 
 * @ejb.bean type="CMP" cmp-version="2.x" name="Flight"
 *           local-jndi-name="airline/FlightLocalHome" view-type="local"
 *           primkey-field="flightId"
 * 
 * @ejb.value-object name="Flight" extends="ejb.flights.entity.FlightLightValue"
 *                   match="*" instantiation="eager"
 * 
 * @ejb.value-object name="FlightLight" match="light"
 * 
 * @ejb.facade type="Stateless" view-type="remote"
 * 
 * 
 * @ejb.finder signature="java.util.Collection findAll()"
 *             result-type-mapping="Local" method-intf="LocalHome" query="SELECT
 *             OBJECT(o) FROM Flight o"
 * 
 * @ejb.finder signature="java.util.Collection findByName(java.lang.String
 *             name)" result-type-mapping="Local" method-intf="LocalHome"
 *             query="SELECT DISTINCT OBJECT(o) FROM Flight o WHERE o.name = ?1"
 *             description="name is not indexed."
 * @ejb.finder signature="java.util.Collection
 *             findByDepartureUtc(java.sql.Timestamp departureUtc)"
 *             result-type-mapping="Local" method-intf="LocalHome" query="SELECT
 *             DISTINCT OBJECT(o) FROM Flight o WHERE o.departureUtc = ?1"
 *             description="departure_utc is not indexed."
 * @ejb.finder signature="java.util.Collection
 *             findByArrivalUtc(java.sql.Timestamp arrivalUtc)"
 *             result-type-mapping="Local" method-intf="LocalHome" query="SELECT
 *             DISTINCT OBJECT(o) FROM Flight o WHERE o.arrivalUtc = ?1"
 *             description="arrival_utc is not indexed."
 * @ejb.persistence table-name="flights"
 * @ejb.transaction type="Required"
 * 
 * @weblogic.data-source-name jdbc_1
 * 
 * @jonas.bean ejb-name="Flight" jndi-name="airline/FlightLocalHome"
 *             cleanup="none"
 * 
 * @jonas.jdbc-mapping jndi-name="jdbc_1" jdbc-table-name="flights"
 *                     automatic_pk="true"
 *  
 */

public abstract class FlightBean implements javax.ejb.EntityBean {

	public void ejbActivate() {
	}

	/**
	 * Creates an instance based on a value object
	 * 
	 * When the client invokes a create method, the EJB container invokes the
	 * ejbCreate method. Typically, an ejbCreate method in an entity bean
	 * performs the following tasks:
	 * <UL>
	 * <LI>Inserts the entity state into the database.</LI>
	 * <LI>Initializes the instance variables.</LI>
	 * <LI>Returns the primary key.</LI>
	 * </UL>
	 * 
	 * @param value
	 *            the value object used to initialise the new instance
	 * @return the primary key of the new instance
	 * 
	 * @ejb.create-method
	 */
	public java.lang.Integer ejbCreate(ejb.flights.entity.FlightLightValue value)
			throws javax.ejb.CreateException {
		// Use XDoclet's GUID generator. Only works for String fields
		// This requires <utilobject includeGUID="true"/> in ejbdoclet.

		setFlightId(value.getFlightId());
		setName(value.getName());
		setDepartureUtc(value.getDepartureUtc());
		setArrivalUtc(value.getArrivalUtc());
		// EJB 2.0 spec says return null for CMP ejbCreate methods.
		return null;
	}

	/**
	 * This create method takes only mandatory (non-nullable) parameters.
	 * 
	 * When the client invokes a create method, the EJB container invokes the
	 * ejbCreate method. Typically, an ejbCreate method in an entity bean
	 * performs the following tasks:
	 * <UL>
	 * <LI>Inserts the entity state into the database.</LI>
	 * <LI>Initializes the instance variables.</LI>
	 * <LI>Returns the primary key.</LI>
	 * </UL>
	 * 
	 * @param flightId
	 *            the flightId value
	 * @param name
	 *            the name value
	 * @param departureUtc
	 *            the departureUtc value
	 * @param arrivalUtc
	 *            the arrivalUtc value
	 * @return the primary key of the new instance
	 * 
	 * @ejb.create-method
	 */
	public java.lang.Integer ejbCreate(java.lang.Integer flightId,
			java.lang.String name, java.sql.Timestamp departureUtc,
			java.sql.Timestamp arrivalUtc) throws javax.ejb.CreateException {
		// Use XDoclet's GUID generator. Only works for String fields
		// This requires <utilobject includeGUID="true"/> in ejbdoclet.

		// Set CMP fields
		setFlightId(flightId);
		setName(name);
		setDepartureUtc(departureUtc);
		setArrivalUtc(arrivalUtc);
		// EJB 2.0 spec says return null for CMP ejbCreate methods.
		return null;
	}

	// No H:\Projet\Middlegen\samples/src/middlegen/cmp20-flights-class-code.txt
	// found.

	public void ejbLoad() {
	}

	public void ejbPassivate() {
	}

	/**
	 * The container invokes this method immediately after it calls ejbCreate.
	 * 
	 * @param value
	 *            the value object used to initialise the new instance
	 */
	public void ejbPostCreate(ejb.flights.entity.FlightLightValue value)
			throws javax.ejb.CreateException {
	}

	/**
	 * The container invokes thos method immediately after it calls ejbCreate.
	 * 
	 * @param flightId
	 *            the flightId value
	 * @param name
	 *            the name value
	 * @param departureUtc
	 *            the departureUtc value
	 * @param arrivalUtc
	 *            the arrivalUtc value
	 */
	public void ejbPostCreate(java.lang.Integer flightId,
			java.lang.String name, java.sql.Timestamp departureUtc,
			java.sql.Timestamp arrivalUtc) throws javax.ejb.CreateException {
		// Set CMR fields
	}

	public void ejbRemove() throws javax.ejb.RemoveException {

	}

	public void ejbStore() {
	}

	/**
	 * Returns the arrivalUtc
	 * 
	 * @todo support OracleClob,OracleBlob on WLS
	 * 
	 * @return the arrivalUtc
	 * 
	 * @ejb.interface-method view-type="local"
	 * @ejb.persistent-field @ejb.persistence column-name="arrival_utc"
	 * @ejb.value-object match="light"
	 * @jonas.cmp-field-jdbc-mapping field-name="arrivalUtc"
	 *                               jdbc-field-name="arrival_utc"
	 */
	public abstract java.sql.Timestamp getArrivalUtc();

	/**
	 * Returns the departureUtc
	 * 
	 * @todo support OracleClob,OracleBlob on WLS
	 * 
	 * @return the departureUtc
	 * 
	 * @ejb.interface-method view-type="local"
	 * @ejb.persistent-field @ejb.persistence column-name="departure_utc"
	 * @ejb.value-object match="light"
	 * @jonas.cmp-field-jdbc-mapping field-name="departureUtc"
	 *                               jdbc-field-name="departure_utc"
	 */
	public abstract java.sql.Timestamp getDepartureUtc();

	/**
	 * Returns the flightId
	 * 
	 * @todo support OracleClob,OracleBlob on WLS
	 * 
	 * @return the flightId
	 * 
	 * @ejb.pk-field @ejb.interface-method view-type="local"
	 * @ejb.persistent-field @ejb.persistence column-name="flight_id"
	 * @ejb.value-object match="light"
	 * @jonas.cmp-field-jdbc-mapping field-name="flightId"
	 *                               jdbc-field-name="flight_id"
	 */
	public abstract java.lang.Integer getFlightId();

	/**
	 * Return the light value object version of this entity.
	 * 
	 * @ejb.interface-method view-type="local"
	 */
	public abstract ejb.flights.entity.FlightLightValue getFlightLightValue();

	/**
	 * Return the value object version of this entity.
	 * 
	 * @ejb.interface-method view-type="local"
	 */
	public abstract ejb.flights.entity.FlightValue getFlightValue();

	/**
	 * Returns the name
	 * 
	 * @todo support OracleClob,OracleBlob on WLS
	 * 
	 * @return the name
	 * 
	 * @ejb.interface-method view-type="local"
	 * @ejb.persistent-field @ejb.persistence column-name="name"
	 * @ejb.value-object match="light"
	 * @jonas.cmp-field-jdbc-mapping field-name="name" jdbc-field-name="name"
	 */
	public abstract java.lang.String getName();

	/**
	 * This is a bi-directional one-to-many relationship CMR method
	 * 
	 * @return a java.util.Collection of related
	 *         ejb.reservations.entity.ReservationLocal.
	 * 
	 * @ejb.interface-method view-type="local"
	 * 
	 * @ejb.relation name="flights-cmp20-reservations-cmp20"
	 *               role-name="flights-cmp20-has-reservations-cmp20"
	 * 
	 * @jboss.relation-mapping style="foreign-key"
	 * 
	 * @ejb.value-object aggregate="ejb.reservations.entity.ReservationLightValue"
	 *                   aggregate-name="reservations"
	 *                   members="ejb.reservations.entity.ReservationLocal"
	 *                   members-name="reservations" match="all"
	 *                   relation="external" type="java.util.Collection"
	 */
	public abstract java.util.Collection getReservations();

	/**
	 * Create and return a value object populated with the data from this bean.
	 * 
	 * Standard method that must be on all Beans for the TreeBuilder to work its
	 * magic.
	 * 
	 * @return Returns a value object containing the data within this bean.
	 * 
	 * @ejb.interface-method view-type="local"
	 */
	public ejb.flights.entity.FlightValue getValueObject() {
		ejb.flights.entity.FlightValue valueObject = new ejb.flights.entity.FlightValue();
		valueObject.setFlightId(getFlightId());
		valueObject.setName(getName());
		valueObject.setDepartureUtc(getDepartureUtc());
		valueObject.setArrivalUtc(getArrivalUtc());
		return valueObject;
	}

	/**
	 * Sets the arrivalUtc
	 * 
	 * @param arrivalUtc
	 *            the new arrivalUtc value
	 * @ejb.interface-method view-type="local"
	 */
	public abstract void setArrivalUtc(java.sql.Timestamp arrivalUtc);

	/**
	 * Sets the departureUtc
	 * 
	 * @param departureUtc
	 *            the new departureUtc value
	 * @ejb.interface-method view-type="local"
	 */
	public abstract void setDepartureUtc(java.sql.Timestamp departureUtc);

	public void setEntityContext(javax.ejb.EntityContext ctx) {
	}

	/**
	 * Sets the flightId
	 * 
	 * @param flightId
	 *            the new flightId value
	 */
	public abstract void setFlightId(java.lang.Integer flightId);

	/**
	 * Set the light value object version of this entity.
	 * 
	 * @ejb.interface-method view-type="local"
	 */
	public abstract void setFlightLightValue(
			ejb.flights.entity.FlightLightValue value);

	/**
	 * Set the value object version of this entity.
	 * 
	 * @ejb.interface-method view-type="local"
	 */
	public abstract void setFlightValue(ejb.flights.entity.FlightValue value);

	/**
	 * Sets the name
	 * 
	 * @param name
	 *            the new name value
	 * @ejb.interface-method view-type="local"
	 * @ejb:transaction type="Never"
	 */
	public abstract void setName(java.lang.String name);

	/**
	 * Sets a collection of related ejb.reservations.entity.ReservationLocal
	 * 
	 * @param a
	 *            collection of related ejb.reservations.entity.ReservationLocal
	 * 
	 * @ejb.interface-method view-type="local"
	 * 
	 * @param reservations
	 *            the new CMR value
	 */
	public abstract void setReservations(java.util.Collection reservations);

	public void unsetEntityContext() {
	}

}